SystemOrganization addCategory: #'Math-Tests-Random'!

TestCase subclass: #BernoulliGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!BernoulliGeneratorTest commentStamp: '<historical>' prior: 0!
A BernoulliGeneratorTest is a test class for testing the behavior of BernoulliGenerator!

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testGenerator
	| g bern |
	g := LCRandom new.
	bern := BernoulliGenerator new.
	self should: [bern generator isKindOf: BernoulliGenerator defaultGenerator].
	bern generator: g.
	self should: [bern generator isKindOf: LCRandom]
	! !

!BernoulliGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testNextYieldsOneOrZero
	| gen |
	gen := BernoulliGenerator fair.
	self should: [ | x | x := gen next.
				(x = 0) or: [x = 1]].! !

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testOneProbabilityGivesOneNext
	| g |
	g := BernoulliGenerator withProbability: 1.
	self should: [g next = 1].! !

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 6/7/2012 12:35'!
testProbabilityIsMutable
	| g |
	g := BernoulliGenerator withProbability: 0.0.
	self should: [g next = 0].
	self should: [g p = 0].
	g p: 1.
	self should: [g next = 1].
	self should: [g p = 1].! !

!BernoulliGeneratorTest methodsFor: 'tests-accessing' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testZeroProbabilityGivesZeroNext
	| g |
	g := BernoulliGenerator withProbability: 0.0.
	self should: [g next = 0]! !

TestCase subclass: #BinomialGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!BinomialGeneratorTest methodsFor: 'tests' stamp: 'WernerKassens 4/27/2014 18:55'!
testBinomialGeneratorConvergesToMean
	"this test may fail one or more assertions, but its purpose is to verify correct convergence of the binomial distribution, should be Normal (np, np(1-p))"
	| gen nums mean|
	mean := Random new next sqrt.
	gen := BinomialGenerator n: 3000 p: mean.
	mean := mean * 3000.
	nums := OrderedCollection new.
	(1 to: 3000) do: [:ea | nums add: gen next].
	self should: [nums min > (mean * 0.8)].
	self should: [nums max < (mean * 1.2)].
	self should: [(nums average - mean) abs < 5 ].
	
	Transcript 
		cr;
		show: 'BinomialGeneratorTest min: ', nums min asString, 
		' max: ', nums max asString, ' mean: ', mean asInteger asString,
		' sample mean: ', nums average asInteger asString; 
		cr. 
! !

!BinomialGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testBinomialGeneratorWithSuccessProbabilityOfOneAlwaysReturnNumberOfTrials
	| g numberOfTrials |
	g := BinomialGenerator new.
	numberOfTrials := 10.
	g
		n: numberOfTrials;
		p: 1.0.
	g generator: LCRandom new.
	self assert: g next = numberOfTrials.
	g generator: LehmerRandom new.
	self assert: g next = numberOfTrials.
	g generator: MersenneTwisterRandom new.
	self assert: g next = numberOfTrials! !

!BinomialGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testBinomialGeneratorWithSuccessProbabilityOfZeroAlwaysReturnZero
	| g numberOfTrials |
	g := BinomialGenerator new.
	numberOfTrials := 10.
	g
		n: numberOfTrials;
		p: 0.0.
	g generator: LCRandom new.
	self assert: g next = 0.
	g generator: LehmerRandom new.
	self assert: g next = 0.
	g generator: MersenneTwisterRandom new.
	self assert: g next = 0! !

TestCase subclass: #ConstantGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!ConstantGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testConstantGenerator
	| g |
	g := ConstantGenerator new.
	g constant: 1.
	self assert: g next = g next.
	self assert: g next = g next.
	self assert: g next = 1.
	
	g := ConstantGenerator constant: 2.
	self assert: g next = 2.! !

TestCase subclass: #ExponentialGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!ExponentialGeneratorTest commentStamp: '<historical>' prior: 0!
An ExponentialGeneratorTest is a test class for testing the behavior of ExponentialGenerator!

!ExponentialGeneratorTest methodsFor: 'tests-as yet unclassified' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testGenerator
	| eg |
	eg := ExponentialGenerator new.
	self should: [eg generator isKindOf: ExponentialGenerator defaultGenerator].
	self should: [eg generator isKindOf: RandomGenerator].
	eg generator: MersenneTwisterRandom new.
	self should: [eg generator isKindOf: MersenneTwisterRandom].
	self should: [eg next isKindOf: Number].! !

!ExponentialGeneratorTest methodsFor: 'tests-as yet unclassified' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testPeekAlwaysAnswersTheSame
	| eg |
	eg := ExponentialGenerator new.
	self should: [eg peek = eg peek].
	self should: [eg peek = eg next].! !

!ExponentialGeneratorTest methodsFor: 'tests-as yet unclassified' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testSampleMeanConvergesToDistributionMean
	"testing a random sample seems suspect. We use a 5% interval here"
	| eg arr |
	eg := ExponentialGenerator mean: 10.
	arr := Array new: 10000.
	(1 to: 10000) do: [:index | arr at: index put: eg next].
	self should: [arr average between: (eg mean * 0.95) and: (eg mean * 1.05) ].! !

TestCase subclass: #GaussianGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!GaussianGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
checkDistributionOf: aGenerator withExpectedMeans: e andExpectedStandardDeviation: sd
	"fixme: this test ignores the standard deviation and occasionally fails"
	| data m |
	data := Set new.
	10000 timesRepeat: [ data add: aGenerator next ].
	m := data average.
	self assert: e - m <= 0.2! !

!GaussianGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 6/6/2012 09:56'!
testDistribution
	| mean standardDeviation g |
	mean := 147.
	standardDeviation := 17.
	g := (GaussianGenerator new).
	g
		mean: mean;
		standardDeviation: standardDeviation.
	g generator: LCRandom new.
	self checkDistributionOf: g withExpectedMeans: mean andExpectedStandardDeviation: standardDeviation.
	"Replicated test in a different place"
	mean := 0.
	standardDeviation := 10 .
	g := (GaussianGenerator new)
		mean: mean;
		standardDeviation: standardDeviation;
		generator: LCRandom new.
	self checkDistributionOf: g withExpectedMeans: mean andExpectedStandardDeviation: standardDeviation.! !

TestCase subclass: #LCRandomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!LCRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testNextBetweenZeroAndOne
	| g |
	g := LCRandom new.
	1000 timesRepeat:
		[ self should: [g next >= 0].
		self should: [g next <= 1]].! !

!LCRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testPeekAlwaysReplyTheSameValue
	| g |
	g := LCRandom new.
	self assert: g peek = g peek! !

!LCRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testPeekAnswersSameAsNext
	| g |
	g := LCRandom new.
	self assert: g peek = g next! !

TestCase subclass: #LehmerRandomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!LehmerRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testNextBetweenZeroAndOne
	| g |
	g := LehmerRandom new.
	1000 timesRepeat:
		[ self should: [g next > 0].
		self should: [g next < 1]].! !

!LehmerRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testPeekAlwaysReplyTheSameValue
	| g |
	g := LehmerRandom new.
	self assert: g peek = g peek! !

!LehmerRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testPeekAnswersSameAsNext
	| g |
	g := LehmerRandom new.
	self assert: g peek = g next! !

TestCase subclass: #MersenneTwisterRandomTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!MersenneTwisterRandomTest methodsFor: 'tests' stamp: 'DanielUber 6/7/2012 13:33'!
testNext10BetweenZeroAndTen
	| g |
	g := MersenneTwisterRandom new.
	1000 timesRepeat:
		[ | value |
		value := g next.
		 self should: [value > 0].
		self should: [value < 10]].! !

!MersenneTwisterRandomTest methodsFor: 'tests' stamp: 'DanielUber 5/31/2012 09:52:48.324'!
testNextBetweenZeroAndOne
	| g |
	g := MersenneTwisterRandom new.
	1000 timesRepeat:
		[ self should: [g next > 0].
		self should: [g next < 1]].! !

!MersenneTwisterRandomTest methodsFor: 'tests' stamp: 'DanielUber 6/7/2012 14:03'!
testNextFloatExcludeUpper
	"this revealed a bug in early versions, excludeUpper was producing between 0 and 2"
	| g sample1 sample2 |
	g := MersenneTwisterRandom new.
	sample1 := Array new: 10000.
	sample2 := Array new: 10000.
	1 to: 10000 do: 
		[:i |
		sample1 at: i put: g nextFloatExcludeUpper.
		sample2 at: i put: (g nextFloatExcludeUpper: 2)].
	self should: [sample1 max < 1].
	self shouldnt: [sample1 max < 0.8].
	self should: [sample2 max < 2].
	self shouldnt: [sample2 max < 1.6].! !

TestCase subclass: #NumberGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!NumberGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 6/6/2012 09:49'!
testPeekAnswersSame
"every subclass should implement some basic behavior."
	NumberGenerator subclasses do: [:ea | |random numberGenerator |
		numberGenerator := ea new.
		10 timesRepeat: [
		random := numberGenerator peek.
		self should: [random = numberGenerator peek].
		self should: [random = numberGenerator next]]]! !

TestCase subclass: #PoissonGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!PoissonGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 6/6/2012 07:08'!
testPeekReturnsSameAsNext
	| poisson random |
	poisson := PoissonGenerator lambda: 0.1.
	random := poisson peek.
	self should: [random = poisson next].

	! !

!PoissonGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 6/6/2012 07:08'!
testPeekReturnsSameTwice
	| poisson random |
	poisson := PoissonGenerator lambda: 0.1.
	random := poisson peek.
	self should: [random = poisson peek].

	! !

!PoissonGeneratorTest methodsFor: 'tests' stamp: 'DanielUber 6/6/2012 07:10'!
testPeekWorksAfterSampling
	| poisson random |
	poisson := PoissonGenerator lambda: 0.1.
	(1 to: 100) do: [:ea | poisson next].
	random := poisson peek.
	self should: [random = poisson peek].
	self should: [random = poisson next].	! !

!PoissonGeneratorTest methodsFor: 'tests' stamp: 'WernerKassens 4/27/2014 18:54'!
testSampleAverageConvergesToLambda
	"law of large numbers"
	| poisson samples |
	samples := OrderedCollection new.
	poisson := PoissonGenerator lambda: (1000 atRandom -1).
	1000 timesRepeat: [samples add: poisson next].
	self should: [samples average >= (poisson lambda * 0.8)].
	self should: [samples average <= (poisson lambda * 1.2)].! !

TestCase subclass: #RandomGeneratorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-Random'!

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 13:30'!
testGeneratorStreamDoesntRespondToContents
	RandomGenerator subclasses do: 
	[:cls | | gen value |
		gen := cls new.
		self should: [gen contents] raise: Error].! !

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 13:24'!
testGeneratorStreamIsReadOnly
	RandomGenerator subclasses do:
		[:cls | |gen|
		gen := cls new.
		self shouldnt: [gen isWriteable].
		self should: [gen isReadable]]! !

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 12:39'!
testGeneratorStreamNeverEnds
	RandomGenerator subclasses do: 
	[:cls | | gen |
		gen := cls new.
		self shouldnt: [gen atEnd]].! !

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 12:41'!
testGeneratorStreamPeekIsNext
	RandomGenerator subclasses do: 
	[:cls | | gen value |
		gen := cls new.
		value := gen peek.
		self should: [gen peek = value].
		self should: [gen next = value]].! !

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 12:40'!
testGeneratorStreamProducesNumbers
	RandomGenerator subclasses do: 
	[:cls | | gen |
		gen := cls new.
		self should: [gen next isNumber].
		self should: [gen next isFloat]].! !

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 12:50'!
testGeneratorStreamProducesRandomNumbers
	RandomGenerator subclasses do: 
	[:cls | | gen value |
		gen := cls new.
		value := gen next.
		self shouldnt: [gen next = value]].! !

!RandomGeneratorTest methodsFor: 'as yet unclassified' stamp: 'DanielUber 6/7/2012 13:28'!
testNextGivesArrayOfNumbers
	RandomGenerator subclasses do:
		[:cls | | gen samples| 
		gen := cls new.
		samples := gen next: 10.
		self should: [samples size = 10].
		self should: [(samples at: 1 ) isFloat].
		self should: [samples asSet size = 10]]! !
